@@ -55,6 +55,8 @@
 
 namespace {
 
+constexpr int MaxAdpcmChannels{2};
+
 /* IMA ADPCM Stepsize table */
 constexpr int IMAStep_size[89] = {
        7,    8,    9,   10,   11,   12,   13,   14,   16,   17,   19,
@@ -99,11 +101,11 @@ constexpr int MSADPCMAdaptionCoeff[7][2] = {
 };
 
 
-void DecodeIMA4Block(ALshort *dst, const al::byte *src, size_t numchans, size_t align)
+void DecodeIMA4Block(int16_t *dst, const al::byte *src, size_t numchans, size_t align)
 {
-    ALint sample[MAX_INPUT_CHANNELS]{};
-    ALint index[MAX_INPUT_CHANNELS]{};
-    ALuint code[MAX_INPUT_CHANNELS]{};
+    int sample[MaxAdpcmChannels]{};
+    int index[MaxAdpcmChannels]{};
+    ALuint code[MaxAdpcmChannels]{};
 
     for(size_t c{0};c < numchans;c++)
     {
@@ -114,7 +116,7 @@ void DecodeIMA4Block(ALshort *dst, const al::byte *src, size_t numchans, size_t
         index[c] = clampi((index[c]^0x8000) - 32768, 0, 88);
         src += 2;
 
-        *(dst++) = static_cast<ALshort>(sample[c]);
+        *(dst++) = static_cast<int16_t>(sample[c]);
     }
 
     for(size_t i{1};i < align;i++)
@@ -140,16 +142,16 @@ void DecodeIMA4Block(ALshort *dst, const al::byte *src, size_t numchans, size_t
             index[c] += IMA4Index_adjust[nibble];
             index[c] = clampi(index[c], 0, 88);
 
-            *(dst++) = static_cast<ALshort>(sample[c]);
+            *(dst++) = static_cast<int16_t>(sample[c]);
         }
     }
 }
 
-void DecodeMSADPCMBlock(ALshort *dst, const al::byte *src, size_t numchans, size_t align)
+void DecodeMSADPCMBlock(int16_t *dst, const al::byte *src, size_t numchans, size_t align)
 {
-    ALubyte blockpred[MAX_INPUT_CHANNELS]{};
-    ALint delta[MAX_INPUT_CHANNELS]{};
-    ALshort samples[MAX_INPUT_CHANNELS][2]{};
+    uint8_t blockpred[MaxAdpcmChannels]{};
+    int delta[MaxAdpcmChannels]{};
+    int16_t samples[MaxAdpcmChannels][2]{};
 
     for(size_t c{0};c < numchans;c++)
     {
@@ -193,25 +195,26 @@ void DecodeMSADPCMBlock(ALshort *dst, const al::byte *src, size_t numchans, size
             else
                 nibble = *(src++) & 0x0f;
 
-            ALint pred{(samples[c][0]*MSADPCMAdaptionCoeff[blockpred[c]][0] +
+            int pred{(samples[c][0]*MSADPCMAdaptionCoeff[blockpred[c]][0] +
                 samples[c][1]*MSADPCMAdaptionCoeff[blockpred[c]][1]) / 256};
             pred += (al::to_integer<int>(nibble^0x08) - 0x08) * delta[c];
             pred  = clampi(pred, -32768, 32767);
 
             samples[c][1] = samples[c][0];
-            samples[c][0] = static_cast<ALshort>(pred);
+            samples[c][0] = static_cast<int16_t>(pred);
 
             delta[c] = (MSADPCMAdaption[al::to_integer<ALubyte>(nibble)] * delta[c]) / 256;
             delta[c] = maxi(16, delta[c]);
 
-            *(dst++) = static_cast<ALshort>(pred);
+            *(dst++) = static_cast<int16_t>(pred);
         }
     }
 }
 
-void Convert_ALshort_ALima4(ALshort *dst, const al::byte *src, size_t numchans, size_t len,
+void Convert_int16_ima4(int16_t *dst, const al::byte *src, size_t numchans, size_t len,
     size_t align)
 {
+    assert(numchans <= MaxAdpcmChannels);
     const size_t byte_align{((align-1)/2 + 4) * numchans};
 
     len /= align;
@@ -223,9 +226,10 @@ void Convert_ALshort_ALima4(ALshort *dst, const al::byte *src, size_t numchans,
     }
 }
 
-void Convert_ALshort_ALmsadpcm(ALshort *dst, const al::byte *src, size_t numchans, size_t len,
+void Convert_int16_msadpcm(int16_t *dst, const al::byte *src, size_t numchans, size_t len,
     size_t align)
 {
+    assert(numchans <= MaxAdpcmChannels);
     const size_t byte_align{((align-2)/2 + 7) * numchans};
 
     len /= align;
@@ -238,22 +242,22 @@ void Convert_ALshort_ALmsadpcm(ALshort *dst, const al::byte *src, size_t numchan
 }
 
 
-ALuint BytesFromUserFmt(UserFmtType type)
+ALuint BytesFromUserFmt(UserFmtType type) noexcept
 {
     switch(type)
     {
-    case UserFmtUByte: return sizeof(ALubyte);
-    case UserFmtShort: return sizeof(ALshort);
-    case UserFmtFloat: return sizeof(ALfloat);
-    case UserFmtDouble: return sizeof(ALdouble);
-    case UserFmtMulaw: return sizeof(ALubyte);
-    case UserFmtAlaw: return sizeof(ALubyte);
+    case UserFmtUByte: return sizeof(uint8_t);
+    case UserFmtShort: return sizeof(int16_t);
+    case UserFmtFloat: return sizeof(float);
+    case UserFmtDouble: return sizeof(double);
+    case UserFmtMulaw: return sizeof(uint8_t);
+    case UserFmtAlaw: return sizeof(uint8_t);
     case UserFmtIMA4: break; /* not handled here */
     case UserFmtMSADPCM: break; /* not handled here */
     }
     return 0;
 }
-ALuint ChannelsFromUserFmt(UserFmtChannels chans)
+ALuint ChannelsFromUserFmt(UserFmtChannels chans, ALuint ambiorder) noexcept
 {
     switch(chans)
     {
@@ -264,13 +268,11 @@ ALuint ChannelsFromUserFmt(UserFmtChannels chans)
     case UserFmtX51: return 6;
     case UserFmtX61: return 7;
     case UserFmtX71: return 8;
-    case UserFmtBFormat2D: return 3;
-    case UserFmtBFormat3D: return 4;
+    case UserFmtBFormat2D: return (ambiorder*2) + 1;
+    case UserFmtBFormat3D: return (ambiorder+1) * (ambiorder+1);
     }
     return 0;
 }
-inline ALuint FrameSizeFromUserFmt(UserFmtChannels chans, UserFmtType type)
-{ return ChannelsFromUserFmt(chans) * BytesFromUserFmt(type); }
 
 
 constexpr ALbitfieldSOFT INVALID_STORAGE_MASK{~unsigned(AL_MAP_READ_BIT_SOFT |
@@ -284,8 +286,7 @@ bool EnsureBuffers(ALCdevice *device, size_t needed)
 {
     size_t count{std::accumulate(device->BufferList.cbegin(), device->BufferList.cend(), size_t{0},
         [](size_t cur, const BufferSubList &sublist) noexcept -> size_t
-        { return cur + static_cast<ALuint>(POPCNT64(sublist.FreeMask)); }
-    )};
+        { return cur + static_cast<ALuint>(PopCount(sublist.FreeMask)); })};
 
     while(needed > count)
     {
@@ -314,7 +315,7 @@ ALbuffer *AllocBuffer(ALCdevice *device)
     );
 
     auto lidx = static_cast<ALuint>(std::distance(device->BufferList.begin(), sublist));
-    auto slidx = static_cast<ALuint>(CTZ64(sublist->FreeMask));
+    auto slidx = static_cast<ALuint>(CountTrailingZeros(sublist->FreeMask));
 
     ALbuffer *buffer{::new (sublist->Buffers + slidx) ALbuffer{}};
 
@@ -389,8 +390,8 @@ const ALchar *NameFromUserFmtType(UserFmtType type)
 {
     switch(type)
     {
-    case UserFmtUByte: return "Unsigned Byte";
-    case UserFmtShort: return "Signed Short";
+    case UserFmtUByte: return "UInt8";
+    case UserFmtShort: return "Int16";
     case UserFmtFloat: return "Float32";
     case UserFmtDouble: return "Float64";
     case UserFmtMulaw: return "muLaw";
@@ -458,23 +459,27 @@ void LoadData(ALCcontext *context, ALbuffer *ALBuf, ALsizei freq, ALuint size,
         SETERR_RETURN(context, AL_INVALID_VALUE,, "Invalid unpack alignment %u for %s samples",
             unpackalign, NameFromUserFmtType(SrcType));
 
+    const ALuint ambiorder{(DstChannels == FmtBFormat2D || DstChannels == FmtBFormat3D) ?
+        ALBuf->UnpackAmbiOrder : 0};
+
     if((access&AL_PRESERVE_DATA_BIT_SOFT))
     {
         /* Can only preserve data with the same format and alignment. */
-        if UNLIKELY(ALBuf->mFmtChannels != DstChannels || ALBuf->OriginalType != SrcType)
+        if UNLIKELY(ALBuf->mBuffer.mChannels != DstChannels || ALBuf->OriginalType != SrcType)
             SETERR_RETURN(context, AL_INVALID_VALUE,, "Preserving data of mismatched format");
         if UNLIKELY(ALBuf->OriginalAlign != align)
             SETERR_RETURN(context, AL_INVALID_VALUE,, "Preserving data of mismatched alignment");
+        if(ALBuf->mBuffer.mAmbiOrder != ambiorder)
+            SETERR_RETURN(context, AL_INVALID_VALUE,, "Preserving data of mismatched order");
     }
 
     /* Convert the input/source size in bytes to sample frames using the unpack
      * block alignment.
      */
-    const ALuint SrcByteAlign{
-        (SrcType == UserFmtIMA4) ? ((align-1)/2 + 4) * ChannelsFromUserFmt(SrcChannels) :
-        (SrcType == UserFmtMSADPCM) ? ((align-2)/2 + 7) * ChannelsFromUserFmt(SrcChannels) :
-        (align * FrameSizeFromUserFmt(SrcChannels, SrcType))
-    };
+    const ALuint SrcByteAlign{ChannelsFromUserFmt(SrcChannels, ambiorder) *
+        ((SrcType == UserFmtIMA4) ? (align-1)/2 + 4 :
+        (SrcType == UserFmtMSADPCM) ? (align-2)/2 + 7 :
+        (align * BytesFromUserFmt(SrcType)))};
     if UNLIKELY((size%SrcByteAlign) != 0)
         SETERR_RETURN(context, AL_INVALID_VALUE,,
             "Data size %d is not a multiple of frame size %d (%d unpack alignment)",
@@ -488,7 +493,7 @@ void LoadData(ALCcontext *context, ALbuffer *ALBuf, ALsizei freq, ALuint size,
     /* Convert the sample frames to the number of bytes needed for internal
      * storage.
      */
-    ALuint NumChannels{ChannelsFromFmt(DstChannels)};
+    ALuint NumChannels{ChannelsFromFmt(DstChannels, ambiorder)};
     ALuint FrameSize{NumChannels * BytesFromFmt(DstType)};
     if UNLIKELY(frames > std::numeric_limits<size_t>::max()/FrameSize)
         SETERR_RETURN(context, AL_OUT_OF_MEMORY,,
@@ -502,53 +507,124 @@ void LoadData(ALCcontext *context, ALbuffer *ALBuf, ALsizei freq, ALuint size,
      * use AL_SIZE to try to get the buffer's play length.
      */
     newsize = RoundUp(newsize, 16);
-    if(newsize != ALBuf->mData.size())
+    if(newsize != ALBuf->mBuffer.mData.size())
     {
         auto newdata = al::vector<al::byte,16>(newsize, al::byte{});
         if((access&AL_PRESERVE_DATA_BIT_SOFT))
         {
-            const size_t tocopy{minz(newdata.size(), ALBuf->mData.size())};
-            std::copy_n(ALBuf->mData.begin(), tocopy, newdata.begin());
+            const size_t tocopy{minz(newdata.size(), ALBuf->mBuffer.mData.size())};
+            std::copy_n(ALBuf->mBuffer.mData.begin(), tocopy, newdata.begin());
         }
-        ALBuf->mData = std::move(newdata);
+        newdata.swap(ALBuf->mBuffer.mData);
     }
 
     if(SrcType == UserFmtIMA4)
     {
         assert(DstType == FmtShort);
-        if(SrcData != nullptr && !ALBuf->mData.empty())
-            Convert_ALshort_ALima4(reinterpret_cast<ALshort*>(ALBuf->mData.data()),
-                SrcData, NumChannels, frames, align);
+        if(SrcData != nullptr && !ALBuf->mBuffer.mData.empty())
+            Convert_int16_ima4(reinterpret_cast<int16_t*>(ALBuf->mBuffer.mData.data()), SrcData,
+                NumChannels, frames, align);
         ALBuf->OriginalAlign = align;
     }
     else if(SrcType == UserFmtMSADPCM)
     {
         assert(DstType == FmtShort);
-        if(SrcData != nullptr && !ALBuf->mData.empty())
-            Convert_ALshort_ALmsadpcm(reinterpret_cast<ALshort*>(ALBuf->mData.data()),
-                SrcData, NumChannels, frames, align);
+        if(SrcData != nullptr && !ALBuf->mBuffer.mData.empty())
+            Convert_int16_msadpcm(reinterpret_cast<int16_t*>(ALBuf->mBuffer.mData.data()), SrcData,
+                NumChannels, frames, align);
         ALBuf->OriginalAlign = align;
     }
     else
     {
         assert(static_cast<long>(SrcType) == static_cast<long>(DstType));
-        if(SrcData != nullptr && !ALBuf->mData.empty())
-            std::copy_n(SrcData, frames*FrameSize, ALBuf->mData.begin());
+        if(SrcData != nullptr && !ALBuf->mBuffer.mData.empty())
+            std::copy_n(SrcData, frames*FrameSize, ALBuf->mBuffer.mData.begin());
         ALBuf->OriginalAlign = 1;
     }
     ALBuf->OriginalSize = size;
     ALBuf->OriginalType = SrcType;
 
-    ALBuf->Frequency = static_cast<ALuint>(freq);
-    ALBuf->mFmtChannels = DstChannels;
-    ALBuf->mFmtType = DstType;
+    ALBuf->mBuffer.mSampleRate = static_cast<ALuint>(freq);
+    ALBuf->mBuffer.mChannels = DstChannels;
+    ALBuf->mBuffer.mType = DstType;
     ALBuf->Access = access;
+    ALBuf->mBuffer.mAmbiOrder = ambiorder;
 
-    ALBuf->SampleLen = frames;
+    ALBuf->mBuffer.mCallback = nullptr;
+    ALBuf->mBuffer.mUserData = nullptr;
+
+    ALBuf->mBuffer.mSampleLen = frames;
     ALBuf->LoopStart = 0;
-    ALBuf->LoopEnd = ALBuf->SampleLen;
+    ALBuf->LoopEnd = ALBuf->mBuffer.mSampleLen;
 }
 
+/** Prepares the buffer to use the specified callback, using the specified format. */
+void PrepareCallback(ALCcontext *context, ALbuffer *ALBuf, ALsizei freq,
+    UserFmtChannels SrcChannels, UserFmtType SrcType, LPALBUFFERCALLBACKTYPESOFT callback,
+    void *userptr)
+{
+    if UNLIKELY(ReadRef(ALBuf->ref) != 0 || ALBuf->MappedAccess != 0)
+        SETERR_RETURN(context, AL_INVALID_OPERATION,, "Modifying callback for in-use buffer %u",
+            ALBuf->id);
+
+    /* Currently no channel configurations need to be converted. */
+    FmtChannels DstChannels{FmtMono};
+    switch(SrcChannels)
+    {
+    case UserFmtMono: DstChannels = FmtMono; break;
+    case UserFmtStereo: DstChannels = FmtStereo; break;
+    case UserFmtRear: DstChannels = FmtRear; break;
+    case UserFmtQuad: DstChannels = FmtQuad; break;
+    case UserFmtX51: DstChannels = FmtX51; break;
+    case UserFmtX61: DstChannels = FmtX61; break;
+    case UserFmtX71: DstChannels = FmtX71; break;
+    case UserFmtBFormat2D: DstChannels = FmtBFormat2D; break;
+    case UserFmtBFormat3D: DstChannels = FmtBFormat3D; break;
+    }
+    if UNLIKELY(static_cast<long>(SrcChannels) != static_cast<long>(DstChannels))
+        SETERR_RETURN(context, AL_INVALID_ENUM,, "Invalid format");
+
+    /* IMA4 and MSADPCM convert to 16-bit short. Not supported with callbacks. */
+    FmtType DstType{FmtUByte};
+    switch(SrcType)
+    {
+    case UserFmtUByte: DstType = FmtUByte; break;
+    case UserFmtShort: DstType = FmtShort; break;
+    case UserFmtFloat: DstType = FmtFloat; break;
+    case UserFmtDouble: DstType = FmtDouble; break;
+    case UserFmtAlaw: DstType = FmtAlaw; break;
+    case UserFmtMulaw: DstType = FmtMulaw; break;
+    case UserFmtIMA4: DstType = FmtShort; break;
+    case UserFmtMSADPCM: DstType = FmtShort; break;
+    }
+    if UNLIKELY(static_cast<long>(SrcType) != static_cast<long>(DstType))
+        SETERR_RETURN(context, AL_INVALID_ENUM,, "Unsupported callback format");
+
+    const ALuint ambiorder{(DstChannels == FmtBFormat2D || DstChannels == FmtBFormat3D) ?
+        ALBuf->UnpackAmbiOrder : 0};
+
+    al::vector<al::byte,16>(FrameSizeFromFmt(DstChannels, DstType, ambiorder) *
+        size_t{BUFFERSIZE + (MAX_RESAMPLER_PADDING>>1)}).swap(ALBuf->mBuffer.mData);
+
+    ALBuf->mBuffer.mCallback = callback;
+    ALBuf->mBuffer.mUserData = userptr;
+
+    ALBuf->OriginalType = SrcType;
+    ALBuf->OriginalSize = 0;
+    ALBuf->OriginalAlign = 1;
+
+    ALBuf->mBuffer.mSampleRate = static_cast<ALuint>(freq);
+    ALBuf->mBuffer.mChannels = DstChannels;
+    ALBuf->mBuffer.mType = DstType;
+    ALBuf->Access = 0;
+    ALBuf->mBuffer.mAmbiOrder = ambiorder;
+
+    ALBuf->mBuffer.mSampleLen = 0;
+    ALBuf->LoopStart = 0;
+    ALBuf->LoopEnd = ALBuf->mBuffer.mSampleLen;
+}
+
+
 struct DecompResult { UserFmtChannels channels; UserFmtType type; };
 al::optional<DecompResult> DecomposeUserFormat(ALenum format)
 {
@@ -626,7 +702,7 @@ al::optional<DecompResult> DecomposeUserFormat(ALenum format)
 } // namespace
 
 
-AL_API ALvoid AL_APIENTRY alGenBuffers(ALsizei n, ALuint *buffers)
+AL_API void AL_APIENTRY alGenBuffers(ALsizei n, ALuint *buffers)
 START_API_FUNC
 {
     ContextRef context{GetContextRef()};
@@ -666,7 +742,7 @@ START_API_FUNC
 }
 END_API_FUNC
 
-AL_API ALvoid AL_APIENTRY alDeleteBuffers(ALsizei n, const ALuint *buffers)
+AL_API void AL_APIENTRY alDeleteBuffers(ALsizei n, const ALuint *buffers)
 START_API_FUNC
 {
     ContextRef context{GetContextRef()};
@@ -726,7 +802,7 @@ START_API_FUNC
 END_API_FUNC
 
 
-AL_API ALvoid AL_APIENTRY alBufferData(ALuint buffer, ALenum format, const ALvoid *data, ALsizei size, ALsizei freq)
+AL_API void AL_APIENTRY alBufferData(ALuint buffer, ALenum format, const ALvoid *data, ALsizei size, ALsizei freq)
 START_API_FUNC
 { alBufferStorageSOFT(buffer, format, data, size, freq, 0); }
 END_API_FUNC
@@ -806,7 +882,7 @@ START_API_FUNC
                 offset, length, buffer);
         else
         {
-            void *retval = albuf->mData.data() + offset;
+            void *retval{albuf->mBuffer.mData.data() + offset};
             albuf->MappedAccess = access;
             albuf->MappedOffset = offset;
             albuf->MappedSize = length;
@@ -873,7 +949,7 @@ START_API_FUNC
 }
 END_API_FUNC
 
-AL_API ALvoid AL_APIENTRY alBufferSubDataSOFT(ALuint buffer, ALenum format, const ALvoid *data, ALsizei offset, ALsizei length)
+AL_API void AL_APIENTRY alBufferSubDataSOFT(ALuint buffer, ALenum format, const ALvoid *data, ALsizei offset, ALsizei length)
 START_API_FUNC
 {
     ContextRef context{GetContextRef()};
@@ -900,13 +976,16 @@ START_API_FUNC
     ALuint align{SanitizeAlignment(usrfmt->type, unpack_align)};
     if UNLIKELY(align < 1)
         context->setError(AL_INVALID_VALUE, "Invalid unpack alignment %u", unpack_align);
-    else if UNLIKELY(long{usrfmt->channels} != long{albuf->mFmtChannels}
+    else if UNLIKELY(long{usrfmt->channels} != long{albuf->mBuffer.mChannels}
         || usrfmt->type != albuf->OriginalType)
         context->setError(AL_INVALID_ENUM, "Unpacking data with mismatched format");
     else if UNLIKELY(align != albuf->OriginalAlign)
         context->setError(AL_INVALID_VALUE,
             "Unpacking data with alignment %u does not match original alignment %u", align,
             albuf->OriginalAlign);
+    else if UNLIKELY(albuf->mBuffer.isBFormat()
+        && albuf->UnpackAmbiOrder != albuf->mBuffer.mAmbiOrder)
+        context->setError(AL_INVALID_VALUE, "Unpacking data with mismatched ambisonic order");
     else if UNLIKELY(albuf->MappedAccess != 0)
         context->setError(AL_INVALID_OPERATION, "Unpacking data into mapped buffer %u", buffer);
     else
@@ -937,16 +1016,16 @@ START_API_FUNC
             size_t byteoff{static_cast<ALuint>(offset)/byte_align * align * frame_size};
             size_t samplen{static_cast<ALuint>(length)/byte_align * align};
 
-            void *dst = albuf->mData.data() + byteoff;
-            if(usrfmt->type == UserFmtIMA4 && albuf->mFmtType == FmtShort)
-                Convert_ALshort_ALima4(static_cast<ALshort*>(dst),
-                    static_cast<const al::byte*>(data), num_chans, samplen, align);
-            else if(usrfmt->type == UserFmtMSADPCM && albuf->mFmtType == FmtShort)
-                Convert_ALshort_ALmsadpcm(static_cast<ALshort*>(dst),
+            void *dst = albuf->mBuffer.mData.data() + byteoff;
+            if(usrfmt->type == UserFmtIMA4 && albuf->mBuffer.mType == FmtShort)
+                Convert_int16_ima4(static_cast<int16_t*>(dst), static_cast<const al::byte*>(data),
+                    num_chans, samplen, align);
+            else if(usrfmt->type == UserFmtMSADPCM && albuf->mBuffer.mType == FmtShort)
+                Convert_int16_msadpcm(static_cast<int16_t*>(dst),
                     static_cast<const al::byte*>(data), num_chans, samplen, align);
             else
             {
-                assert(long{usrfmt->type} == long{albuf->mFmtType});
+                assert(long{usrfmt->type} == long{albuf->mBuffer.mType});
                 memcpy(dst, data, size_t{samplen} * frame_size);
             }
         }
@@ -1097,7 +1176,7 @@ START_API_FUNC
         else if UNLIKELY(value != AL_FUMA_SOFT && value != AL_ACN_SOFT)
             context->setError(AL_INVALID_VALUE, "Invalid unpack ambisonic layout %04x", value);
         else
-            albuf->AmbiLayout = value;
+            albuf->mBuffer.mAmbiLayout = static_cast<AmbiLayout>(value);
         break;
 
     case AL_AMBISONIC_SCALING_SOFT:
@@ -1107,7 +1186,14 @@ START_API_FUNC
         else if UNLIKELY(value != AL_FUMA_SOFT && value != AL_SN3D_SOFT && value != AL_N3D_SOFT)
             context->setError(AL_INVALID_VALUE, "Invalid unpack ambisonic scaling %04x", value);
         else
-            albuf->AmbiScaling = value;
+            albuf->mBuffer.mAmbiScaling = static_cast<AmbiScaling>(value);
+        break;
+
+    case AL_UNPACK_AMBISONIC_ORDER_SOFT:
+        if UNLIKELY(value < 1 || value > 14)
+            context->setError(AL_INVALID_VALUE, "Invalid unpack ambisonic order %d", value);
+        else
+            albuf->UnpackAmbiOrder = static_cast<ALuint>(value);
         break;
 
     default:
@@ -1147,6 +1233,7 @@ START_API_FUNC
         case AL_PACK_BLOCK_ALIGNMENT_SOFT:
         case AL_AMBISONIC_LAYOUT_SOFT:
         case AL_AMBISONIC_SCALING_SOFT:
+        case AL_UNPACK_AMBISONIC_ORDER_SOFT:
             alBufferi(buffer, param, values[0]);
             return;
         }
@@ -1170,7 +1257,7 @@ START_API_FUNC
             context->setError(AL_INVALID_OPERATION, "Modifying in-use buffer %u's loop points",
                 buffer);
         else if UNLIKELY(values[0] < 0 || values[0] >= values[1]
-            || static_cast<ALuint>(values[1]) > albuf->SampleLen)
+            || static_cast<ALuint>(values[1]) > albuf->mBuffer.mSampleLen)
             context->setError(AL_INVALID_VALUE, "Invalid loop point range %d -> %d on buffer %u",
                 values[0], values[1], buffer);
         else
@@ -1187,7 +1274,7 @@ START_API_FUNC
 END_API_FUNC
 
 
-AL_API ALvoid AL_APIENTRY alGetBufferf(ALuint buffer, ALenum param, ALfloat *value)
+AL_API void AL_APIENTRY alGetBufferf(ALuint buffer, ALenum param, ALfloat *value)
 START_API_FUNC
 {
     ContextRef context{GetContextRef()};
@@ -1259,7 +1346,7 @@ START_API_FUNC
 END_API_FUNC
 
 
-AL_API ALvoid AL_APIENTRY alGetBufferi(ALuint buffer, ALenum param, ALint *value)
+AL_API void AL_APIENTRY alGetBufferi(ALuint buffer, ALenum param, ALint *value)
 START_API_FUNC
 {
     ContextRef context{GetContextRef()};
@@ -1275,7 +1362,7 @@ START_API_FUNC
     else switch(param)
     {
     case AL_FREQUENCY:
-        *value = static_cast<ALint>(albuf->Frequency);
+        *value = static_cast<ALint>(albuf->mBuffer.mSampleRate);
         break;
 
     case AL_BITS:
@@ -1287,7 +1374,7 @@ START_API_FUNC
         break;
 
     case AL_SIZE:
-        *value = static_cast<ALint>(albuf->SampleLen * albuf->frameSizeFromFmt());
+        *value = static_cast<ALint>(albuf->mBuffer.mSampleLen * albuf->frameSizeFromFmt());
         break;
 
     case AL_UNPACK_BLOCK_ALIGNMENT_SOFT:
@@ -1299,11 +1386,15 @@ START_API_FUNC
         break;
 
     case AL_AMBISONIC_LAYOUT_SOFT:
-        *value = albuf->AmbiLayout;
+        *value = static_cast<int>(albuf->mBuffer.mAmbiLayout);
         break;
 
     case AL_AMBISONIC_SCALING_SOFT:
-        *value = albuf->AmbiScaling;
+        *value = static_cast<int>(albuf->mBuffer.mAmbiScaling);
+        break;
+
+    case AL_UNPACK_AMBISONIC_ORDER_SOFT:
+        *value = static_cast<int>(albuf->UnpackAmbiOrder);
         break;
 
     default:
@@ -1348,6 +1439,7 @@ START_API_FUNC
     case AL_PACK_BLOCK_ALIGNMENT_SOFT:
     case AL_AMBISONIC_LAYOUT_SOFT:
     case AL_AMBISONIC_SCALING_SOFT:
+    case AL_UNPACK_AMBISONIC_ORDER_SOFT:
         alGetBufferi(buffer, param, values);
         return;
     }
@@ -1376,35 +1468,112 @@ START_API_FUNC
 END_API_FUNC
 
 
-ALuint BytesFromFmt(FmtType type) noexcept
+AL_API void AL_APIENTRY alBufferCallbackSOFT(ALuint buffer, ALenum format, ALsizei freq,
+    LPALBUFFERCALLBACKTYPESOFT callback, ALvoid *userptr, ALbitfieldSOFT flags)
+START_API_FUNC
 {
-    switch(type)
+    ContextRef context{GetContextRef()};
+    if UNLIKELY(!context) return;
+
+    ALCdevice *device{context->mDevice.get()};
+    std::lock_guard<std::mutex> _{device->BufferLock};
+
+    ALbuffer *albuf = LookupBuffer(device, buffer);
+    if UNLIKELY(!albuf)
+        context->setError(AL_INVALID_NAME, "Invalid buffer ID %u", buffer);
+    else if UNLIKELY(freq < 1)
+        context->setError(AL_INVALID_VALUE, "Invalid sample rate %d", freq);
+    else if UNLIKELY(callback == nullptr)
+        context->setError(AL_INVALID_VALUE, "NULL callback");
+    else if UNLIKELY(flags != 0)
+        context->setError(AL_INVALID_VALUE, "Invalid callback flags 0x%x", flags);
+    else
     {
-    case FmtUByte: return sizeof(ALubyte);
-    case FmtShort: return sizeof(ALshort);
-    case FmtFloat: return sizeof(ALfloat);
-    case FmtDouble: return sizeof(ALdouble);
-    case FmtMulaw: return sizeof(ALubyte);
-    case FmtAlaw: return sizeof(ALubyte);
+        auto usrfmt = DecomposeUserFormat(format);
+        if UNLIKELY(!usrfmt)
+            context->setError(AL_INVALID_ENUM, "Invalid format 0x%04x", format);
+        else
+            PrepareCallback(context.get(), albuf, freq, usrfmt->channels, usrfmt->type, callback,
+                userptr);
     }
-    return 0;
 }
-ALuint ChannelsFromFmt(FmtChannels chans) noexcept
+END_API_FUNC
+
+AL_API void AL_APIENTRY alGetBufferPtrSOFT(ALuint buffer, ALenum param, ALvoid **value)
+START_API_FUNC
 {
-    switch(chans)
+    ContextRef context{GetContextRef()};
+    if UNLIKELY(!context) return;
+
+    ALCdevice *device{context->mDevice.get()};
+    std::lock_guard<std::mutex> _{device->BufferLock};
+    ALbuffer *albuf = LookupBuffer(device, buffer);
+    if UNLIKELY(!albuf)
+        context->setError(AL_INVALID_NAME, "Invalid buffer ID %u", buffer);
+    else if UNLIKELY(!value)
+        context->setError(AL_INVALID_VALUE, "NULL pointer");
+    else switch(param)
     {
-    case FmtMono: return 1;
-    case FmtStereo: return 2;
-    case FmtRear: return 2;
-    case FmtQuad: return 4;
-    case FmtX51: return 6;
-    case FmtX61: return 7;
-    case FmtX71: return 8;
-    case FmtBFormat2D: return 3;
-    case FmtBFormat3D: return 4;
+    case AL_BUFFER_CALLBACK_FUNCTION_SOFT:
+        *value = reinterpret_cast<void*>(albuf->mBuffer.mCallback);
+        break;
+    case AL_BUFFER_CALLBACK_USER_PARAM_SOFT:
+        *value = albuf->mBuffer.mUserData;
+        break;
+
+    default:
+        context->setError(AL_INVALID_ENUM, "Invalid buffer pointer property 0x%04x", param);
     }
-    return 0;
 }
+END_API_FUNC
+
+AL_API void AL_APIENTRY alGetBuffer3PtrSOFT(ALuint buffer, ALenum param, ALvoid **value1, ALvoid **value2, ALvoid **value3)
+START_API_FUNC
+{
+    ContextRef context{GetContextRef()};
+    if UNLIKELY(!context) return;
+
+    ALCdevice *device{context->mDevice.get()};
+    std::lock_guard<std::mutex> _{device->BufferLock};
+    if UNLIKELY(LookupBuffer(device, buffer) == nullptr)
+        context->setError(AL_INVALID_NAME, "Invalid buffer ID %u", buffer);
+    else if UNLIKELY(!value1 || !value2 || !value3)
+        context->setError(AL_INVALID_VALUE, "NULL pointer");
+    else switch(param)
+    {
+    default:
+        context->setError(AL_INVALID_ENUM, "Invalid buffer 3-pointer property 0x%04x", param);
+    }
+}
+END_API_FUNC
+
+AL_API void AL_APIENTRY alGetBufferPtrvSOFT(ALuint buffer, ALenum param, ALvoid **values)
+START_API_FUNC
+{
+    switch(param)
+    {
+    case AL_BUFFER_CALLBACK_FUNCTION_SOFT:
+    case AL_BUFFER_CALLBACK_USER_PARAM_SOFT:
+        alGetBufferPtrSOFT(buffer, param, values);
+        return;
+    }
+
+    ContextRef context{GetContextRef()};
+    if UNLIKELY(!context) return;
+
+    ALCdevice *device{context->mDevice.get()};
+    std::lock_guard<std::mutex> _{device->BufferLock};
+    if UNLIKELY(LookupBuffer(device, buffer) == nullptr)
+        context->setError(AL_INVALID_NAME, "Invalid buffer ID %u", buffer);
+    else if UNLIKELY(!values)
+        context->setError(AL_INVALID_VALUE, "NULL pointer");
+    else switch(param)
+    {
+    default:
+        context->setError(AL_INVALID_ENUM, "Invalid buffer pointer-vector property 0x%04x", param);
+    }
+}
+END_API_FUNC
 
 
 BufferSubList::~BufferSubList()
@@ -1412,7 +1581,7 @@ BufferSubList::~BufferSubList()
     uint64_t usemask{~FreeMask};
     while(usemask)
     {
-        ALsizei idx{CTZ64(usemask)};
+        const ALsizei idx{CountTrailingZeros(usemask)};
         al::destroy_at(Buffers+idx);
         usemask &= ~(1_u64 << idx);
     }
