@@ -5,6 +5,7 @@
 #include <atomic>
 #include <cstddef>
 #include <iterator>
+#include <limits>
 
 #include "AL/al.h"
 #include "AL/alc.h"
@@ -13,6 +14,7 @@
 #include "almalloc.h"
 #include "alnumeric.h"
 #include "alu.h"
+#include "math_defs.h"
 #include "vector.h"
 
 struct ALbuffer;
@@ -34,68 +36,68 @@ struct ALbufferlistitem {
 
 struct ALsource {
     /** Source properties. */
-    ALfloat   Pitch;
-    ALfloat   Gain;
-    ALfloat   OuterGain;
-    ALfloat   MinGain;
-    ALfloat   MaxGain;
-    ALfloat   InnerAngle;
-    ALfloat   OuterAngle;
-    ALfloat   RefDistance;
-    ALfloat   MaxDistance;
-    ALfloat   RolloffFactor;
-    std::array<ALfloat,3> Position;
-    std::array<ALfloat,3> Velocity;
-    std::array<ALfloat,3> Direction;
-    std::array<ALfloat,3> OrientAt;
-    std::array<ALfloat,3> OrientUp;
-    bool HeadRelative;
-    bool Looping;
-    DistanceModel mDistanceModel;
-    Resampler mResampler;
-    DirectMode DirectChannels;
-    SpatializeMode mSpatialize;
+    float Pitch{1.0f};
+    float Gain{1.0f};
+    float OuterGain{0.0f};
+    float MinGain{0.0f};
+    float MaxGain{1.0f};
+    float InnerAngle{360.0f};
+    float OuterAngle{360.0f};
+    float RefDistance{1.0f};
+    float MaxDistance{std::numeric_limits<float>::max()};
+    float RolloffFactor{1.0f};
+    std::array<float,3> Position{{0.0f, 0.0f, 0.0f}};
+    std::array<float,3> Velocity{{0.0f, 0.0f, 0.0f}};
+    std::array<float,3> Direction{{0.0f, 0.0f, 0.0f}};
+    std::array<float,3> OrientAt{{0.0f, 0.0f, -1.0f}};
+    std::array<float,3> OrientUp{{0.0f, 1.0f,  0.0f}};
+    bool HeadRelative{false};
+    bool Looping{false};
+    DistanceModel mDistanceModel{DistanceModel::Default};
+    Resampler mResampler{ResamplerDefault};
+    DirectMode DirectChannels{DirectMode::Off};
+    SpatializeMode mSpatialize{SpatializeMode::Auto};
 
-    bool DryGainHFAuto;
-    bool WetGainAuto;
-    bool WetGainHFAuto;
-    ALfloat OuterGainHF;
+    bool DryGainHFAuto{true};
+    bool WetGainAuto{true};
+    bool WetGainHFAuto{true};
+    float OuterGainHF{1.0f};
 
-    ALfloat AirAbsorptionFactor;
-    ALfloat RoomRolloffFactor;
-    ALfloat DopplerFactor;
+    float AirAbsorptionFactor{0.0f};
+    float RoomRolloffFactor{0.0f};
+    float DopplerFactor{1.0f};
 
     /* NOTE: Stereo pan angles are specified in radians, counter-clockwise
      * rather than clockwise.
      */
-    std::array<ALfloat,2> StereoPan;
+    std::array<float,2> StereoPan{{Deg2Rad( 30.0f), Deg2Rad(-30.0f)}};
 
-    ALfloat Radius;
+    float Radius{0.0f};
 
     /** Direct filter and auxiliary send info. */
     struct {
-        ALfloat Gain;
-        ALfloat GainHF;
-        ALfloat HFReference;
-        ALfloat GainLF;
-        ALfloat LFReference;
+        float Gain;
+        float GainHF;
+        float HFReference;
+        float GainLF;
+        float LFReference;
     } Direct;
     struct SendData {
         ALeffectslot *Slot;
-        ALfloat Gain;
-        ALfloat GainHF;
-        ALfloat HFReference;
-        ALfloat GainLF;
-        ALfloat LFReference;
+        float Gain;
+        float GainHF;
+        float HFReference;
+        float GainLF;
+        float LFReference;
     };
-    al::vector<SendData> Send;
+    std::array<SendData,MAX_SENDS> Send;
 
     /**
      * Last user-specified offset, and the offset type (bytes, samples, or
      * seconds).
      */
-    ALdouble Offset{0.0};
-    ALenum   OffsetType{AL_NONE};
+    double Offset{0.0};
+    ALenum OffsetType{AL_NONE};
 
     /** Source type (static, streaming, or undetermined) */
     ALenum SourceType{AL_UNDETERMINED};
@@ -117,11 +119,13 @@ struct ALsource {
     ALuint id{0};
 
 
-    ALsource(ALuint num_sends);
+    ALsource();
     ~ALsource();
 
     ALsource(const ALsource&) = delete;
     ALsource& operator=(const ALsource&) = delete;
+
+    DISABLE_ALLOC()
 };
 
 void UpdateAllSourceProps(ALCcontext *context);
