@@ -23,7 +23,7 @@ public:
     void clear() noexcept { mLpZ1 = mLpZ2 = mApZ1 = 0.0f; }
     void process(const al::span<const Real> input, Real *hpout, Real *lpout);
 
-    void applyHfScale(const al::span<Real> samples, const Real hfscale);
+    void processHfScale(const al::span<Real> samples, const Real hfscale);
 
     /* The all-pass portion of the band splitter. Applies the same phase shift
      * without splitting the signal. Note that each use of this method is
