@@ -73,22 +73,22 @@ struct LoadedHrtf {
 /* Data set limits must be the same as or more flexible than those defined in
  * the makemhr utility.
  */
-#define MIN_FD_COUNT                 (1)
-#define MAX_FD_COUNT                 (16)
+#define MIN_FD_COUNT                 1
+#define MAX_FD_COUNT                 16
 
-#define MIN_FD_DISTANCE              (50)
-#define MAX_FD_DISTANCE              (2500)
+#define MIN_FD_DISTANCE              50
+#define MAX_FD_DISTANCE              2500
 
-#define MIN_EV_COUNT                 (5)
-#define MAX_EV_COUNT                 (181)
+#define MIN_EV_COUNT                 5
+#define MAX_EV_COUNT                 181
 
-#define MIN_AZ_COUNT                 (1)
-#define MAX_AZ_COUNT                 (255)
+#define MIN_AZ_COUNT                 1
+#define MAX_AZ_COUNT                 255
 
 #define MAX_HRIR_DELAY               (HRTF_HISTORY_LENGTH-1)
 
 #define HRIR_DELAY_FRACBITS 2
-#define HRIR_DELAY_FRACONE (1<<HRIR_DELAY_FRACBITS)
+#define HRIR_DELAY_FRACONE  (1<<HRIR_DELAY_FRACBITS)
 #define HRIR_DELAY_FRACHALF (HRIR_DELAY_FRACONE>>1)
 
 static_assert(MAX_HRIR_DELAY*HRIR_DELAY_FRACONE < 256, "MAX_HRIR_DELAY or DELAY_FRAC too large");
@@ -96,10 +96,11 @@ static_assert(MAX_HRIR_DELAY*HRIR_DELAY_FRACONE < 256, "MAX_HRIR_DELAY or DELAY_
 constexpr ALchar magicMarker00[8]{'M','i','n','P','H','R','0','0'};
 constexpr ALchar magicMarker01[8]{'M','i','n','P','H','R','0','1'};
 constexpr ALchar magicMarker02[8]{'M','i','n','P','H','R','0','2'};
+constexpr ALchar magicMarker03[8]{'M','i','n','P','H','R','0','3'};
 
 /* First value for pass-through coefficients (remaining are 0), used for omni-
  * directional sounds. */
-constexpr ALfloat PassthruCoeff{0.707106781187f/*sqrt(0.5)*/};
+constexpr float PassthruCoeff{0.707106781187f/*sqrt(0.5)*/};
 
 std::mutex LoadedHrtfLock;
 al::vector<LoadedHrtf> LoadedHrtfs;
@@ -210,7 +211,7 @@ IdxBlend CalcAzIndex(ALuint azcount, float az)
  * and azimuth in radians. The coefficients are normalized.
  */
 void GetHrtfCoeffs(const HrtfStore *Hrtf, float elevation, float azimuth, float distance,
-    float spread, HrirArray &coeffs, ALuint (&delays)[2])
+    float spread, HrirArray &coeffs, const al::span<ALuint,2> delays)
 {
     const float dirfact{1.0f - (spread / al::MathDefs<float>::Tau())};
 
@@ -223,7 +224,7 @@ void GetHrtfCoeffs(const HrtfStore *Hrtf, float elevation, float azimuth, float
         ++field;
     }
 
-    /* Claculate the elevation indinces. */
+    /* Calculate the elevation indices. */
     const auto elev0 = CalcEvIndex(field->evCount, elevation);
     const size_t elev1_idx{minu(elev0.idx+1, field->evCount-1)};
     const size_t ir0offset{Hrtf->elev[ebase + elev0.idx].irOffset};
@@ -276,12 +277,10 @@ void GetHrtfCoeffs(const HrtfStore *Hrtf, float elevation, float azimuth, float
 
 
 std::unique_ptr<DirectHrtfState> DirectHrtfState::Create(size_t num_chans)
-{
-    return std::unique_ptr<DirectHrtfState>{new (FamCount{num_chans}) DirectHrtfState{num_chans}};
-}
+{ return std::unique_ptr<DirectHrtfState>{new(FamCount(num_chans)) DirectHrtfState{num_chans}}; }
 
-void BuildBFormatHrtf(const HrtfStore *Hrtf, DirectHrtfState *state,
-    const al::span<const AngularPoint> AmbiPoints, const float (*AmbiMatrix)[MAX_AMBI_CHANNELS],
+void DirectHrtfState::build(const HrtfStore *Hrtf, const al::span<const AngularPoint> AmbiPoints,
+    const float (*AmbiMatrix)[MAX_AMBI_CHANNELS],
     const al::span<const float,MAX_AMBI_ORDER+1> AmbiOrderHFGain)
 {
     using double2 = std::array<double,2>;
@@ -290,35 +289,43 @@ void BuildBFormatHrtf(const HrtfStore *Hrtf, DirectHrtfState *state,
         ALuint ldelay, rdelay;
     };
 
-    /* Set this to true for dual-band HRTF processing. May require better
-     * calculation of the new IR length to deal with the head and tail
-     * generated by the HF scaling.
-     */
-    static constexpr bool DualBand{true};
+    const double xover_norm{400.0 / Hrtf->sampleRate};
+    for(size_t i{0};i < mChannels.size();++i)
+    {
+        const size_t order{AmbiIndex::OrderFromChannel[i]};
+        mChannels[i].mSplitter.init(static_cast<float>(xover_norm));
+        mChannels[i].mHfScale = AmbiOrderHFGain[order];
+    }
 
-    ALuint min_delay{HRTF_HISTORY_LENGTH*HRIR_DELAY_FRACONE};
-    ALuint max_delay{0};
+    ALuint min_delay{HRTF_HISTORY_LENGTH*HRIR_DELAY_FRACONE}, max_delay{0};
     al::vector<ImpulseResponse> impres; impres.reserve(AmbiPoints.size());
     auto calc_res = [Hrtf,&max_delay,&min_delay](const AngularPoint &pt) -> ImpulseResponse
     {
-        auto CalcClosestEvIndex = [](ALuint evcount, float ev) -> ALuint
-        {
-            ev = (al::MathDefs<float>::Pi()*0.5f + ev) * static_cast<float>(evcount-1) /
-                al::MathDefs<float>::Pi();
-            return minu(float2uint(ev+0.5f), evcount-1);
-        };
-        auto CalcClosestAzIndex = [](ALuint azcount, float az) -> ALuint
-        {
-            az = (al::MathDefs<float>::Tau()+az) * static_cast<float>(azcount) /
-                al::MathDefs<float>::Tau();
-            return float2uint(az+0.5f) % azcount;
-        };
-
         auto &field = Hrtf->field[0];
-        const size_t elevIdx{CalcClosestEvIndex(field.evCount, pt.Elev.value)};
-        const size_t azIdx{CalcClosestAzIndex(Hrtf->elev[elevIdx].azCount, pt.Azim.value)};
-        const size_t irOffset{Hrtf->elev[elevIdx].irOffset + azIdx};
+        const auto elev0 = CalcEvIndex(field.evCount, pt.Elev.value);
+        const size_t elev1_idx{minu(elev0.idx+1, field.evCount-1)};
+        const size_t ir0offset{Hrtf->elev[elev0.idx].irOffset};
+        const size_t ir1offset{Hrtf->elev[elev1_idx].irOffset};
 
+        const auto az0 = CalcAzIndex(Hrtf->elev[elev0.idx].azCount, pt.Azim.value);
+        const auto az1 = CalcAzIndex(Hrtf->elev[elev1_idx].azCount, pt.Azim.value);
+
+        const size_t idx[4]{
+            ir0offset + az0.idx,
+            ir0offset + ((az0.idx+1) % Hrtf->elev[elev0.idx].azCount),
+            ir1offset + az1.idx,
+            ir1offset + ((az1.idx+1) % Hrtf->elev[elev1_idx].azCount)
+        };
+
+        const std::array<double,4> blend{{
+            (1.0-elev0.blend) * (1.0-az0.blend),
+            (1.0-elev0.blend) * (    az0.blend),
+            (    elev0.blend) * (1.0-az1.blend),
+            (    elev0.blend) * (    az1.blend)
+        }};
+
+        /* The largest blend factor serves as the closest HRIR. */
+        const size_t irOffset{idx[std::max_element(blend.begin(), blend.end()) - blend.begin()]};
         ImpulseResponse res{Hrtf->coeffs[irOffset],
             Hrtf->delays[irOffset][0], Hrtf->delays[irOffset][1]};
 
@@ -331,118 +338,43 @@ void BuildBFormatHrtf(const HrtfStore *Hrtf, DirectHrtfState *state,
     auto hrir_delay_round = [](const ALuint d) noexcept -> ALuint
     { return (d+HRIR_DELAY_FRACHALF) >> HRIR_DELAY_FRACBITS; };
 
-    /* For dual-band processing, add a 16-sample delay to compensate for the HF
-     * scale on the minimum-phase response.
-     */
-    static constexpr ALuint base_delay{DualBand ? 16 : 0};
-    const double xover_norm{400.0 / Hrtf->sampleRate};
-    BandSplitterR<double> splitter{xover_norm};
-
-    auto tmpres = al::vector<std::array<double2,HRIR_LENGTH>>(state->Coeffs.size());
-    auto tmpflt = al::vector<std::array<double,HRIR_LENGTH*4>>(3);
-    const al::span<double,HRIR_LENGTH*4> tempir{tmpflt[2].data(), tmpflt[2].size()};
+    auto tmpres = al::vector<std::array<double2,HRIR_LENGTH>>(mChannels.size());
     for(size_t c{0u};c < AmbiPoints.size();++c)
     {
         const HrirArray &hrir{impres[c].hrir};
-        const ALuint ldelay{hrir_delay_round(impres[c].ldelay-min_delay) + base_delay};
-        const ALuint rdelay{hrir_delay_round(impres[c].rdelay-min_delay) + base_delay};
+        const ALuint ldelay{hrir_delay_round(impres[c].ldelay - min_delay)};
+        const ALuint rdelay{hrir_delay_round(impres[c].rdelay - min_delay)};
 
-        if /*constexpr*/(!DualBand)
+        for(size_t i{0u};i < mChannels.size();++i)
         {
-            /* For single-band decoding, apply the HF scale to the response. */
-            for(size_t i{0u};i < state->Coeffs.size();++i)
+            const double mult{AmbiMatrix[c][i]};
+            const size_t numirs{HRIR_LENGTH - maxz(ldelay, rdelay)};
+            size_t lidx{ldelay}, ridx{rdelay};
+            for(size_t j{0};j < numirs;++j)
             {
-                const size_t order{AmbiIndex::OrderFromChannel[i]};
-                const double mult{double{AmbiOrderHFGain[order]} * AmbiMatrix[c][i]};
-                const ALuint numirs{HRIR_LENGTH - maxu(ldelay, rdelay)};
-                ALuint lidx{ldelay}, ridx{rdelay};
-                for(ALuint j{0};j < numirs;++j)
-                {
-                    tmpres[i][lidx++][0] += hrir[j][0] * mult;
-                    tmpres[i][ridx++][1] += hrir[j][1] * mult;
-                }
+                tmpres[i][lidx++][0] += hrir[j][0] * mult;
+                tmpres[i][ridx++][1] += hrir[j][1] * mult;
             }
-            continue;
-        }
-
-        /* For dual-band processing, the HRIR needs to be split into low and
-         * high frequency responses. The band-splitter alone creates frequency-
-         * dependent phase-shifts, which is not ideal. To counteract it,
-         * combine it with a backwards phase-shift.
-         */
-
-        /* Load the (left) HRIR backwards, into a temp buffer with padding. */
-        std::fill(tempir.begin(), tempir.end(), 0.0);
-        std::transform(hrir.crbegin(), hrir.crend(), tempir.begin(),
-            [](const float2 &ir) noexcept -> double { return ir[0]; });
-
-        /* Apply the all-pass on the reversed signal and reverse the resulting
-         * sample array. This produces the forward response with a backwards
-         * phase-shift (+n degrees becomes -n degrees).
-         */
-        splitter.applyAllpass({tempir.data(), tempir.size()});
-        std::reverse(tempir.begin(), tempir.end());
-
-        /* Now apply the band-splitter. This applies the normal phase-shift,
-         * which cancels out with the backwards phase-shift to get the original
-         * phase on the split signal.
-         */
-        splitter.clear();
-        splitter.process(tempir, tmpflt[0].data(), tmpflt[1].data());
-
-        /* Apply left ear response with delay and HF scale. */
-        for(size_t i{0u};i < state->Coeffs.size();++i)
-        {
-            const double mult{AmbiMatrix[c][i]};
-            const double hfgain{AmbiOrderHFGain[AmbiIndex::OrderFromChannel[i]]};
-            size_t j{tmpflt[0].size()-HRIR_LENGTH - ldelay};
-            for(size_t lidx{0};lidx < HRIR_LENGTH;++lidx,++j)
-                tmpres[i][lidx][0] += (tmpflt[0][j]*hfgain + tmpflt[1][j]) * mult;
-        }
-
-        /* Now run the same process on the right HRIR. */
-        std::fill(tempir.begin(), tempir.end(), 0.0);
-        std::transform(hrir.crbegin(), hrir.crend(), tempir.begin(),
-            [](const float2 &ir) noexcept -> double { return ir[1]; });
-
-        splitter.applyAllpass({tempir.data(), tempir.size()});
-        std::reverse(tempir.begin(), tempir.end());
-
-        splitter.clear();
-        splitter.process(tempir, tmpflt[0].data(), tmpflt[1].data());
-
-        for(size_t i{0u};i < state->Coeffs.size();++i)
-        {
-            const double mult{AmbiMatrix[c][i]};
-            const double hfgain{AmbiOrderHFGain[AmbiIndex::OrderFromChannel[i]]};
-            size_t j{tmpflt[0].size()-HRIR_LENGTH - rdelay};
-            for(size_t ridx{0};ridx < HRIR_LENGTH;++ridx,++j)
-                tmpres[i][ridx][1] += (tmpflt[0][j]*hfgain + tmpflt[1][j]) * mult;
         }
     }
-    tmpflt.clear();
     impres.clear();
 
-    for(size_t i{0u};i < state->Coeffs.size();++i)
+    for(size_t i{0u};i < mChannels.size();++i)
     {
         auto copy_arr = [](const double2 &in) noexcept -> float2
         { return float2{{static_cast<float>(in[0]), static_cast<float>(in[1])}}; };
-        std::transform(tmpres[i].cbegin(), tmpres[i].cend(), state->Coeffs[i].begin(),
+        std::transform(tmpres[i].cbegin(), tmpres[i].cend(), mChannels[i].mCoeffs.begin(),
             copy_arr);
     }
     tmpres.clear();
 
-    max_delay -= min_delay;
-    /* Increase the IR size by double the base delay with dual-band processing
-     * to account for the head and tail from the HF response scale.
-     */
-    const ALuint irsize{minu(Hrtf->irSize + base_delay*2, HRIR_LENGTH)};
-    const ALuint max_length{minu(hrir_delay_round(max_delay) + irsize, HRIR_LENGTH)};
+    max_delay = hrir_delay_round(max_delay - min_delay);
+    const ALuint max_length{minu(max_delay + Hrtf->irSize, HRIR_LENGTH)};
 
-    TRACE("Skipped delay: %.2f, max delay: %.2f, new FIR length: %u\n",
+    TRACE("Skipped delay: %.2f, new max delay: %.2f, FIR length: %u\n",
         min_delay/double{HRIR_DELAY_FRACONE}, max_delay/double{HRIR_DELAY_FRACONE},
         max_length);
-    state->IrSize = max_length;
+    mIrSize = max_length;
 }
 
 
@@ -551,7 +483,7 @@ ALushort GetLE_ALushort(std::istream &data)
     return static_cast<ALushort>(ret);
 }
 
-ALint GetLE_ALint24(std::istream &data)
+int GetLE_ALint24(std::istream &data)
 {
     int ret = data.get();
     ret |= data.get() << 8;
@@ -561,11 +493,11 @@ ALint GetLE_ALint24(std::istream &data)
 
 ALuint GetLE_ALuint(std::istream &data)
 {
-    int ret = data.get();
-    ret |= data.get() << 8;
-    ret |= data.get() << 16;
-    ret |= data.get() << 24;
-    return static_cast<ALuint>(ret);
+    ALuint ret{static_cast<ALuint>(data.get())};
+    ret |= static_cast<ALuint>(data.get()) << 8;
+    ret |= static_cast<ALuint>(data.get()) << 16;
+    ret |= static_cast<ALuint>(data.get()) << 24;
+    return ret;
 }
 
 std::unique_ptr<HrtfStore> LoadHrtf00(std::istream &data, const char *filename)
@@ -963,7 +895,7 @@ std::unique_ptr<HrtfStore> LoadHrtf02(std::istream &data, const char *filename)
             elevs__end = std::copy_backward(elevs_src, elevs_src+field.evCount, elevs__end);
             return ebase + field.evCount;
         };
-        std::accumulate(fields.cbegin(), fields.cend(), ptrdiff_t{0}, copy_azs);
+        (void)std::accumulate(fields.cbegin(), fields.cend(), ptrdiff_t{0}, copy_azs);
         assert(elevs_.begin() == elevs__end);
 
         /* Reestablish the IR offset for each elevation index, given the new
@@ -998,7 +930,7 @@ std::unique_ptr<HrtfStore> LoadHrtf02(std::istream &data, const char *filename)
 
             return ebase + field.evCount;
         };
-        std::accumulate(fields.cbegin(), fields.cend(), ptrdiff_t{0}, copy_irs);
+        (void)std::accumulate(fields.cbegin(), fields.cend(), ptrdiff_t{0}, copy_irs);
         assert(coeffs_.begin() == coeffs_end);
         assert(delays_.begin() == delays_end);
 
@@ -1012,6 +944,175 @@ std::unique_ptr<HrtfStore> LoadHrtf02(std::istream &data, const char *filename)
         {elevs.data(), elevs.size()}, coeffs.data(), delays.data(), filename);
 }
 
+std::unique_ptr<HrtfStore> LoadHrtf03(std::istream &data, const char *filename)
+{
+    constexpr ALubyte ChanType_LeftOnly{0};
+    constexpr ALubyte ChanType_LeftRight{1};
+
+    ALuint rate{GetLE_ALuint(data)};
+    ALubyte channelType{GetLE_ALubyte(data)};
+    ALushort irSize{GetLE_ALubyte(data)};
+    ALubyte fdCount{GetLE_ALubyte(data)};
+    if(!data || data.eof())
+    {
+        ERR("Failed reading %s\n", filename);
+        return nullptr;
+    }
+
+    if(channelType > ChanType_LeftRight)
+    {
+        ERR("Unsupported channel type: %d\n", channelType);
+        return nullptr;
+    }
+
+    if(irSize < MIN_IR_LENGTH || irSize > HRIR_LENGTH)
+    {
+        ERR("Unsupported HRIR size, irSize=%d (%d to %d)\n", irSize, MIN_IR_LENGTH, HRIR_LENGTH);
+        return nullptr;
+    }
+    if(fdCount < 1 || fdCount > MAX_FD_COUNT)
+    {
+        ERR("Unsupported number of field-depths: fdCount=%d (%d to %d)\n", fdCount, MIN_FD_COUNT,
+            MAX_FD_COUNT);
+        return nullptr;
+    }
+
+    auto fields = al::vector<HrtfStore::Field>(fdCount);
+    auto elevs = al::vector<HrtfStore::Elevation>{};
+    for(size_t f{0};f < fdCount;f++)
+    {
+        const ALushort distance{GetLE_ALushort(data)};
+        const ALubyte evCount{GetLE_ALubyte(data)};
+        if(!data || data.eof())
+        {
+            ERR("Failed reading %s\n", filename);
+            return nullptr;
+        }
+
+        if(distance < MIN_FD_DISTANCE || distance > MAX_FD_DISTANCE)
+        {
+            ERR("Unsupported field distance[%zu]=%d (%d to %d millimeters)\n", f, distance,
+                MIN_FD_DISTANCE, MAX_FD_DISTANCE);
+            return nullptr;
+        }
+        if(evCount < MIN_EV_COUNT || evCount > MAX_EV_COUNT)
+        {
+            ERR("Unsupported elevation count: evCount[%zu]=%d (%d to %d)\n", f, evCount,
+                MIN_EV_COUNT, MAX_EV_COUNT);
+            return nullptr;
+        }
+
+        fields[f].distance = distance / 1000.0f;
+        fields[f].evCount = evCount;
+        if(f > 0 && fields[f].distance > fields[f-1].distance)
+        {
+            ERR("Field distance[%zu] is not before previous (%f <= %f)\n", f, fields[f].distance,
+                fields[f-1].distance);
+            return nullptr;
+        }
+
+        const size_t ebase{elevs.size()};
+        elevs.resize(ebase + evCount);
+        for(auto &elev : al::span<HrtfStore::Elevation>(elevs.data()+ebase, evCount))
+            elev.azCount = GetLE_ALubyte(data);
+        if(!data || data.eof())
+        {
+            ERR("Failed reading %s\n", filename);
+            return nullptr;
+        }
+
+        for(size_t e{0};e < evCount;e++)
+        {
+            if(elevs[ebase+e].azCount < MIN_AZ_COUNT || elevs[ebase+e].azCount > MAX_AZ_COUNT)
+            {
+                ERR("Unsupported azimuth count: azCount[%zu][%zu]=%d (%d to %d)\n", f, e,
+                    elevs[ebase+e].azCount, MIN_AZ_COUNT, MAX_AZ_COUNT);
+                return nullptr;
+            }
+        }
+    }
+
+    elevs[0].irOffset = 0;
+    std::partial_sum(elevs.cbegin(), elevs.cend(), elevs.begin(),
+        [](const HrtfStore::Elevation &last, const HrtfStore::Elevation &cur)
+            -> HrtfStore::Elevation
+        {
+            return HrtfStore::Elevation{cur.azCount,
+                static_cast<ALushort>(last.azCount + last.irOffset)};
+        });
+    const auto irTotal = static_cast<ALushort>(elevs.back().azCount + elevs.back().irOffset);
+
+    auto coeffs = al::vector<HrirArray>(irTotal, HrirArray{});
+    auto delays = al::vector<ubyte2>(irTotal);
+    if(channelType == ChanType_LeftOnly)
+    {
+        for(auto &hrir : coeffs)
+        {
+            for(auto &val : al::span<float2>{hrir.data(), irSize})
+                val[0] = static_cast<float>(GetLE_ALint24(data)) / 8388608.0f;
+        }
+        for(auto &val : delays)
+            val[0] = GetLE_ALubyte(data);
+        if(!data || data.eof())
+        {
+            ERR("Failed reading %s\n", filename);
+            return nullptr;
+        }
+        for(size_t i{0};i < irTotal;++i)
+        {
+            if(delays[i][0] > MAX_HRIR_DELAY<<HRIR_DELAY_FRACBITS)
+            {
+                ERR("Invalid delays[%zu][0]: %f (%d)\n", i,
+                    delays[i][0] / float{HRIR_DELAY_FRACONE}, MAX_HRIR_DELAY);
+                return nullptr;
+            }
+        }
+
+        /* Mirror the left ear responses to the right ear. */
+        MirrorLeftHrirs({elevs.data(), elevs.size()}, coeffs.data(), delays.data());
+    }
+    else if(channelType == ChanType_LeftRight)
+    {
+        for(auto &hrir : coeffs)
+        {
+            for(auto &val : al::span<float2>{hrir.data(), irSize})
+            {
+                val[0] = static_cast<float>(GetLE_ALint24(data)) / 8388608.0f;
+                val[1] = static_cast<float>(GetLE_ALint24(data)) / 8388608.0f;
+            }
+        }
+        for(auto &val : delays)
+        {
+            val[0] = GetLE_ALubyte(data);
+            val[1] = GetLE_ALubyte(data);
+        }
+        if(!data || data.eof())
+        {
+            ERR("Failed reading %s\n", filename);
+            return nullptr;
+        }
+
+        for(size_t i{0};i < irTotal;++i)
+        {
+            if(delays[i][0] > MAX_HRIR_DELAY<<HRIR_DELAY_FRACBITS)
+            {
+                ERR("Invalid delays[%zu][0]: %f (%d)\n", i,
+                    delays[i][0] / float{HRIR_DELAY_FRACONE}, MAX_HRIR_DELAY);
+                return nullptr;
+            }
+            if(delays[i][1] > MAX_HRIR_DELAY<<HRIR_DELAY_FRACBITS)
+            {
+                ERR("Invalid delays[%zu][1]: %f (%d)\n", i,
+                    delays[i][1] / float{HRIR_DELAY_FRACONE}, MAX_HRIR_DELAY);
+                return nullptr;
+            }
+        }
+    }
+
+    return CreateHrtfStore(rate, irSize, {fields.data(), fields.size()},
+        {elevs.data(), elevs.size()}, coeffs.data(), delays.data(), filename);
+}
+
 
 bool checkName(const std::string &name)
 {
@@ -1178,7 +1279,7 @@ al::vector<std::string> EnumerateHrtf(const char *devname)
     return list;
 }
 
-HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALuint devrate)
+HrtfStorePtr GetLoadedHrtf(const std::string &name, const char *devname, const ALuint devrate)
 {
     std::lock_guard<std::mutex> _{EnumeratedHrtfLock};
     auto entry_iter = std::find_if(EnumeratedHrtfs.cbegin(), EnumeratedHrtfs.cend(),
@@ -1197,14 +1298,14 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
         HrtfStore *hrtf{handle->mEntry.get()};
         if(hrtf && hrtf->sampleRate == devrate)
         {
-            hrtf->IncRef();
-            return hrtf;
+            hrtf->add_ref();
+            return HrtfStorePtr{hrtf};
         }
         ++handle;
     }
 
     std::unique_ptr<std::istream> stream;
-    ALint residx{};
+    int residx{};
     char ch{};
     if(sscanf(fname.c_str(), "!%d%c", &residx, &ch) == 2 && ch == '_')
     {
@@ -1215,12 +1316,12 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
             ERR("Could not get resource %u, %s\n", residx, name.c_str());
             return nullptr;
         }
-        stream = al::make_unique<idstream>(res.begin(), res.end());
+        stream = std::make_unique<idstream>(res.begin(), res.end());
     }
     else
     {
         TRACE("Loading %s...\n", fname.c_str());
-        auto fstr = al::make_unique<al::ifstream>(fname.c_str(), std::ios::binary);
+        auto fstr = std::make_unique<al::ifstream>(fname.c_str(), std::ios::binary);
         if(!fstr->is_open())
         {
             ERR("Could not open %s\n", fname.c_str());
@@ -1230,10 +1331,15 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
     }
 
     std::unique_ptr<HrtfStore> hrtf;
-    char magic[sizeof(magicMarker02)];
+    char magic[sizeof(magicMarker03)];
     stream->read(magic, sizeof(magic));
-    if(stream->gcount() < static_cast<std::streamsize>(sizeof(magicMarker02)))
+    if(stream->gcount() < static_cast<std::streamsize>(sizeof(magicMarker03)))
         ERR("%s data is too short (%zu bytes)\n", name.c_str(), stream->gcount());
+    else if(memcmp(magic, magicMarker03, sizeof(magicMarker03)) == 0)
+    {
+        TRACE("Detected data set format v3\n");
+        hrtf = LoadHrtf03(*stream, name.c_str());
+    }
     else if(memcmp(magic, magicMarker02, sizeof(magicMarker02)) == 0)
     {
         TRACE("Detected data set format v2\n");
@@ -1261,6 +1367,8 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
 
     if(hrtf->sampleRate != devrate)
     {
+        TRACE("Resampling HRTF %s (%uhz -> %uhz)\n", name.c_str(), hrtf->sampleRate, devrate);
+
         /* Calculate the last elevation's index and get the total IR count. */
         const size_t lastEv{std::accumulate(hrtf->field, hrtf->field+hrtf->fdCount, size_t{0},
             [](const size_t curval, const HrtfStore::Field &field) noexcept -> size_t
@@ -1316,7 +1424,7 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
         {
             ubyte2 &delays = const_cast<ubyte2&>(hrtf->delays[i]);
             for(size_t j{0};j < 2;++j)
-                delays[j] = static_cast<ALubyte>(float2int(new_delays[i][j] * delay_scale));
+                delays[j] = static_cast<ALubyte>(float2int(new_delays[i][j]*delay_scale + 0.5f));
         }
 
         /* Scale the IR size for the new sample rate and update the stored
@@ -1337,20 +1445,20 @@ HrtfStore *GetLoadedHrtf(const std::string &name, const char *devname, const ALu
         hrtf->sampleRate, hrtf->irSize);
     handle = LoadedHrtfs.emplace(handle, LoadedHrtf{fname, std::move(hrtf)});
 
-    return handle->mEntry.get();
+    return HrtfStorePtr{handle->mEntry.get()};
 }
 
 
-void HrtfStore::IncRef()
+void HrtfStore::add_ref()
 {
     auto ref = IncrementRef(mRef);
-    TRACE("HrtfEntry %p increasing refcount to %u\n", decltype(std::declval<void*>()){this}, ref);
+    TRACE("HrtfStore %p increasing refcount to %u\n", decltype(std::declval<void*>()){this}, ref);
 }
 
-void HrtfStore::DecRef()
+void HrtfStore::release()
 {
     auto ref = DecrementRef(mRef);
-    TRACE("HrtfEntry %p decreasing refcount to %u\n", decltype(std::declval<void*>()){this}, ref);
+    TRACE("HrtfStore %p decreasing refcount to %u\n", decltype(std::declval<void*>()){this}, ref);
     if(ref == 0)
     {
         std::lock_guard<std::mutex> _{LoadedHrtfLock};
