@@ -10,13 +10,22 @@
 #include "alu.h"
 #include "hrtf.h"
 #include "defs.h"
+#include "bsinc_defs.h"
 #include "hrtfbase.h"
 
+struct NEONTag;
+struct LerpTag;
+struct BSincTag;
+struct FastBSincTag;
+
 
 namespace {
 
-inline void ApplyCoeffs(float2 *RESTRICT Values, const ALuint IrSize, const HrirArray &Coeffs,
-    const float left, const float right)
+constexpr ALuint FracPhaseBitDiff{MixerFracBits - BSincPhaseBits};
+constexpr ALuint FracPhaseDiffOne{1 << FracPhaseBitDiff};
+
+inline void ApplyCoeffs(float2 *RESTRICT Values, const uint_fast32_t IrSize,
+    const HrirArray &Coeffs, const float left, const float right)
 {
     float32x4_t leftright4;
     {
@@ -27,7 +36,7 @@ inline void ApplyCoeffs(float2 *RESTRICT Values, const ALuint IrSize, const Hrir
     }
 
     ASSUME(IrSize >= MIN_IR_LENGTH);
-    for(ALuint c{0};c < IrSize;c += 2)
+    for(size_t c{0};c < IrSize;c += 2)
     {
         float32x4_t vals = vld1q_f32(&Values[c][0]);
         float32x4_t coefs = vld1q_f32(&Coeffs[c][0]);
@@ -41,12 +50,12 @@ inline void ApplyCoeffs(float2 *RESTRICT Values, const ALuint IrSize, const Hrir
 } // namespace
 
 template<>
-const ALfloat *Resample_<LerpTag,NEONTag>(const InterpState*, const ALfloat *RESTRICT src,
-    ALuint frac, ALuint increment, const al::span<float> dst)
+const float *Resample_<LerpTag,NEONTag>(const InterpState*, const float *RESTRICT src, ALuint frac,
+    ALuint increment, const al::span<float> dst)
 {
     const int32x4_t increment4 = vdupq_n_s32(static_cast<int>(increment*4));
-    const float32x4_t fracOne4 = vdupq_n_f32(1.0f/FRACTIONONE);
-    const int32x4_t fracMask4 = vdupq_n_s32(FRACTIONMASK);
+    const float32x4_t fracOne4 = vdupq_n_f32(1.0f/MixerFracOne);
+    const int32x4_t fracMask4 = vdupq_n_s32(MixerFracMask);
     alignas(16) ALuint pos_[4], frac_[4];
     int32x4_t pos4, frac4;
 
@@ -55,8 +64,7 @@ const ALfloat *Resample_<LerpTag,NEONTag>(const InterpState*, const ALfloat *RES
     pos4 = vld1q_s32(reinterpret_cast<int*>(pos_));
 
     auto dst_iter = dst.begin();
-    const auto aligned_end = (dst.size()&~3u) + dst_iter;
-    while(dst_iter != aligned_end)
+    for(size_t todo{dst.size()>>2};todo;--todo)
     {
         const int pos0{vgetq_lane_s32(pos4, 0)};
         const int pos1{vgetq_lane_s32(pos4, 1)};
@@ -74,28 +82,28 @@ const ALfloat *Resample_<LerpTag,NEONTag>(const InterpState*, const ALfloat *RES
         dst_iter += 4;
 
         frac4 = vaddq_s32(frac4, increment4);
-        pos4 = vaddq_s32(pos4, vshrq_n_s32(frac4, FRACTIONBITS));
+        pos4 = vaddq_s32(pos4, vshrq_n_s32(frac4, MixerFracBits));
         frac4 = vandq_s32(frac4, fracMask4);
     }
 
-    if(dst_iter != dst.end())
+    if(size_t todo{dst.size()&3})
     {
         src += static_cast<ALuint>(vgetq_lane_s32(pos4, 0));
         frac = static_cast<ALuint>(vgetq_lane_s32(frac4, 0));
 
         do {
-            *(dst_iter++) = lerp(src[0], src[1], static_cast<float>(frac) * (1.0f/FRACTIONONE));
+            *(dst_iter++) = lerp(src[0], src[1], static_cast<float>(frac) * (1.0f/MixerFracOne));
 
             frac += increment;
-            src  += frac>>FRACTIONBITS;
-            frac &= FRACTIONMASK;
-        } while(dst_iter != dst.end());
+            src  += frac>>MixerFracBits;
+            frac &= MixerFracMask;
+        } while(--todo);
     }
-    return dst.begin();
+    return dst.data();
 }
 
 template<>
-const ALfloat *Resample_<BSincTag,NEONTag>(const InterpState *state, const ALfloat *RESTRICT src,
+const float *Resample_<BSincTag,NEONTag>(const InterpState *state, const float *RESTRICT src,
     ALuint frac, ALuint increment, const al::span<float> dst)
 {
     const float *const filter{state->bsinc.filter};
@@ -106,11 +114,8 @@ const ALfloat *Resample_<BSincTag,NEONTag>(const InterpState *state, const ALflo
     for(float &out_sample : dst)
     {
         // Calculate the phase index and factor.
-#define FRAC_PHASE_BITDIFF (FRACTIONBITS-BSINC_PHASE_BITS)
-        const ALuint pi{frac >> FRAC_PHASE_BITDIFF};
-        const float pf{static_cast<float>(frac & ((1<<FRAC_PHASE_BITDIFF)-1)) *
-            (1.0f/(1<<FRAC_PHASE_BITDIFF))};
-#undef FRAC_PHASE_BITDIFF
+        const ALuint pi{frac >> FracPhaseBitDiff};
+        const float pf{static_cast<float>(frac & (FracPhaseDiffOne-1)) * (1.0f/FracPhaseDiffOne)};
 
         // Apply the scale and phase interpolated filter.
         float32x4_t r4{vdupq_n_f32(0.0f)};
@@ -126,9 +131,8 @@ const ALfloat *Resample_<BSincTag,NEONTag>(const InterpState *state, const ALflo
             do {
                 /* f = ((fil + sf*scd) + pf*(phd + sf*spd)) */
                 const float32x4_t f4 = vmlaq_f32(
-                    vmlaq_f32(vld1q_f32(fil), sf4, vld1q_f32(scd)),
-                    pf4, vmlaq_f32(vld1q_f32(phd), sf4, vld1q_f32(spd)));
-                fil += 4; scd += 4; phd += 4; spd += 4;
+                    vmlaq_f32(vld1q_f32(&fil[j]), sf4, vld1q_f32(&scd[j])),
+                    pf4, vmlaq_f32(vld1q_f32(&phd[j]), sf4, vld1q_f32(&spd[j])));
                 /* r += f*src */
                 r4 = vmlaq_f32(r4, f4, vld1q_f32(&src[j]));
                 j += 4;
@@ -138,15 +142,15 @@ const ALfloat *Resample_<BSincTag,NEONTag>(const InterpState *state, const ALflo
         out_sample = vget_lane_f32(vadd_f32(vget_low_f32(r4), vget_high_f32(r4)), 0);
 
         frac += increment;
-        src  += frac>>FRACTIONBITS;
-        frac &= FRACTIONMASK;
+        src  += frac>>MixerFracBits;
+        frac &= MixerFracMask;
     }
-    return dst.begin();
+    return dst.data();
 }
 
 template<>
-const ALfloat *Resample_<FastBSincTag,NEONTag>(const InterpState *state,
-    const ALfloat *RESTRICT src, ALuint frac, ALuint increment, const al::span<float> dst)
+const float *Resample_<FastBSincTag,NEONTag>(const InterpState *state,
+    const float *RESTRICT src, ALuint frac, ALuint increment, const al::span<float> dst)
 {
     const float *const filter{state->bsinc.filter};
     const size_t m{state->bsinc.m};
@@ -155,11 +159,8 @@ const ALfloat *Resample_<FastBSincTag,NEONTag>(const InterpState *state,
     for(float &out_sample : dst)
     {
         // Calculate the phase index and factor.
-#define FRAC_PHASE_BITDIFF (FRACTIONBITS-BSINC_PHASE_BITS)
-        const ALuint pi{frac >> FRAC_PHASE_BITDIFF};
-        const float pf{static_cast<float>(frac & ((1<<FRAC_PHASE_BITDIFF)-1)) *
-            (1.0f/(1<<FRAC_PHASE_BITDIFF))};
-#undef FRAC_PHASE_BITDIFF
+        const ALuint pi{frac >> FracPhaseBitDiff};
+        const float pf{static_cast<float>(frac & (FracPhaseDiffOne-1)) * (1.0f/FracPhaseDiffOne)};
 
         // Apply the phase interpolated filter.
         float32x4_t r4{vdupq_n_f32(0.0f)};
@@ -172,20 +173,20 @@ const ALfloat *Resample_<FastBSincTag,NEONTag>(const InterpState *state,
 
             do {
                 /* f = fil + pf*phd */
-                const float32x4_t f4 = vmlaq_f32(vld1q_f32(fil), pf4, vld1q_f32(phd));
+                const float32x4_t f4 = vmlaq_f32(vld1q_f32(&fil[j]), pf4, vld1q_f32(&phd[j]));
                 /* r += f*src */
                 r4 = vmlaq_f32(r4, f4, vld1q_f32(&src[j]));
-                fil += 4; phd += 4; j += 4;
+                j += 4;
             } while(--td);
         }
         r4 = vaddq_f32(r4, vrev64q_f32(r4));
         out_sample = vget_lane_f32(vadd_f32(vget_low_f32(r4), vget_high_f32(r4)), 0);
 
         frac += increment;
-        src  += frac>>FRACTIONBITS;
-        frac &= FRACTIONMASK;
+        src  += frac>>MixerFracBits;
+        frac &= MixerFracMask;
     }
-    return dst.begin();
+    return dst.data();
 }
 
 
@@ -213,41 +214,40 @@ template<>
 void Mix_<NEONTag>(const al::span<const float> InSamples, const al::span<FloatBufferLine> OutBuffer,
     float *CurrentGains, const float *TargetGains, const size_t Counter, const size_t OutPos)
 {
-    const ALfloat delta{(Counter > 0) ? 1.0f / static_cast<ALfloat>(Counter) : 0.0f};
-    const bool reached_target{InSamples.size() >= Counter};
-    const auto min_end = reached_target ? InSamples.begin() + Counter : InSamples.end();
-    const auto aligned_end = minz(static_cast<uintptr_t>(min_end-InSamples.begin()+3) & ~3u,
-        InSamples.size()) + InSamples.begin();
+    const float delta{(Counter > 0) ? 1.0f / static_cast<float>(Counter) : 0.0f};
+    const auto min_len = minz(Counter, InSamples.size());
+    const auto aligned_len = minz((min_len+3) & ~size_t{3}, InSamples.size()) - min_len;
+
     for(FloatBufferLine &output : OutBuffer)
     {
-        ALfloat *RESTRICT dst{al::assume_aligned<16>(output.data()+OutPos)};
-        ALfloat gain{*CurrentGains};
-        const ALfloat diff{*TargetGains - gain};
+        float *RESTRICT dst{al::assume_aligned<16>(output.data()+OutPos)};
+        float gain{*CurrentGains};
+        const float step{(*TargetGains-gain) * delta};
 
-        auto in_iter = InSamples.begin();
-        if(std::fabs(diff) > std::numeric_limits<float>::epsilon())
+        size_t pos{0};
+        if(!(std::fabs(step) > std::numeric_limits<float>::epsilon()))
+            gain = *TargetGains;
+        else
         {
-            const ALfloat step{diff * delta};
-            ALfloat step_count{0.0f};
+            float step_count{0.0f};
             /* Mix with applying gain steps in aligned multiples of 4. */
-            if(ptrdiff_t todo{(min_end-in_iter) >> 2})
+            if(size_t todo{(min_len-pos) >> 2})
             {
                 const float32x4_t four4{vdupq_n_f32(4.0f)};
                 const float32x4_t step4{vdupq_n_f32(step)};
                 const float32x4_t gain4{vdupq_n_f32(gain)};
-                float32x4_t step_count4{vsetq_lane_f32(0.0f,
-                    vsetq_lane_f32(1.0f,
-                    vsetq_lane_f32(2.0f,
-                    vsetq_lane_f32(3.0f, vdupq_n_f32(0.0f), 3),
-                    2), 1), 0
-                )};
+                float32x4_t step_count4{vdupq_n_f32(0.0f)};
+                step_count4 = vsetq_lane_f32(1.0f, step_count4, 1);
+                step_count4 = vsetq_lane_f32(2.0f, step_count4, 2);
+                step_count4 = vsetq_lane_f32(3.0f, step_count4, 3);
+
                 do {
-                    const float32x4_t val4 = vld1q_f32(in_iter);
-                    float32x4_t dry4 = vld1q_f32(dst);
+                    const float32x4_t val4 = vld1q_f32(&InSamples[pos]);
+                    float32x4_t dry4 = vld1q_f32(&dst[pos]);
                     dry4 = vmlaq_f32(dry4, val4, vmlaq_f32(gain4, step4, step_count4));
                     step_count4 = vaddq_f32(step_count4, four4);
-                    vst1q_f32(dst, dry4);
-                    in_iter += 4; dst += 4;
+                    vst1q_f32(&dst[pos], dry4);
+                    pos += 4;
                 } while(--todo);
                 /* NOTE: step_count4 now represents the next four counts after
                  * the last four mixed samples, so the lowest element
@@ -256,69 +256,38 @@ void Mix_<NEONTag>(const al::span<const float> InSamples, const al::span<FloatBu
                 step_count = vgetq_lane_f32(step_count4, 0);
             }
             /* Mix with applying left over gain steps that aren't aligned multiples of 4. */
-            while(in_iter != min_end)
+            for(size_t leftover{min_len&3};leftover;++pos,--leftover)
             {
-                *(dst++) += *(in_iter++) * (gain + step*step_count);
+                dst[pos] += InSamples[pos] * (gain + step*step_count);
                 step_count += 1.0f;
             }
-            if(reached_target)
+            if(pos == Counter)
                 gain = *TargetGains;
             else
                 gain += step*step_count;
-            *CurrentGains = gain;
 
             /* Mix until pos is aligned with 4 or the mix is done. */
-            while(in_iter != aligned_end)
-                *(dst++) += *(in_iter++) * gain;
+            for(size_t leftover{aligned_len&3};leftover;++pos,--leftover)
+                dst[pos] += InSamples[pos] * gain;
         }
+        *CurrentGains = gain;
         ++CurrentGains;
         ++TargetGains;
 
-        if(!(std::fabs(gain) > GAIN_SILENCE_THRESHOLD))
+        if(!(std::fabs(gain) > GainSilenceThreshold))
             continue;
-        if(ptrdiff_t todo{(InSamples.end()-in_iter) >> 2})
+        if(size_t todo{(InSamples.size()-pos) >> 2})
         {
             const float32x4_t gain4 = vdupq_n_f32(gain);
             do {
-                const float32x4_t val4 = vld1q_f32(in_iter);
-                float32x4_t dry4 = vld1q_f32(dst);
+                const float32x4_t val4 = vld1q_f32(&InSamples[pos]);
+                float32x4_t dry4 = vld1q_f32(&dst[pos]);
                 dry4 = vmlaq_f32(dry4, val4, gain4);
-                vst1q_f32(dst, dry4);
-                in_iter += 4; dst += 4;
+                vst1q_f32(&dst[pos], dry4);
+                pos += 4;
             } while(--todo);
         }
-        while(in_iter != InSamples.end())
-            *(dst++) += *(in_iter++) * gain;
-    }
-}
-
-template<>
-void MixRow_<NEONTag>(const al::span<float> OutBuffer, const al::span<const float> Gains,
-    const float *InSamples, const size_t InStride)
-{
-    for(const ALfloat gain : Gains)
-    {
-        const ALfloat *RESTRICT input{InSamples};
-        InSamples += InStride;
-
-        if(!(std::fabs(gain) > GAIN_SILENCE_THRESHOLD))
-            continue;
-
-        auto out_iter = OutBuffer.begin();
-        if(size_t todo{OutBuffer.size() >> 2})
-        {
-            const float32x4_t gain4{vdupq_n_f32(gain)};
-            do {
-                const float32x4_t val4 = vld1q_f32(input);
-                float32x4_t dry4 = vld1q_f32(out_iter);
-                dry4 = vmlaq_f32(dry4, val4, gain4);
-                vst1q_f32(out_iter, dry4);
-                out_iter += 4; input += 4;
-            } while(--todo);
-        }
-
-        auto do_mix = [gain](const float cur, const float src) noexcept -> float
-        { return cur + src*gain; };
-        std::transform(out_iter, OutBuffer.end(), input, out_iter, do_mix);
+        for(size_t leftover{(InSamples.size()-pos)&3};leftover;++pos,--leftover)
+            dst[pos] += InSamples[pos] * gain;
     }
 }
