@@ -11,10 +11,12 @@
 
 namespace al {
 
-backend_exception::backend_exception(ALCenum code, const char *msg, ...) : mErrorCode{code}
+/* Defined here to avoid inlining it. */
+base_exception::~base_exception() { }
+
+void base_exception::setMessage(const char* msg, std::va_list args)
 {
-    va_list args, args2;
-    va_start(args, msg);
+    std::va_list args2;
     va_copy(args2, args);
     int msglen{std::vsnprintf(nullptr, 0, msg, args)};
     if LIKELY(msglen > 0)
@@ -24,7 +26,6 @@ backend_exception::backend_exception(ALCenum code, const char *msg, ...) : mErro
         mMessage.pop_back();
     }
     va_end(args2);
-    va_end(args);
 }
 
 } // namespace al
