@@ -446,6 +446,7 @@ MainWindow::MainWindow(QWidget *parent) :
     connect(ui->pulseAdjLatencyCheckBox, &QCheckBox::stateChanged, this, &MainWindow::enableApplyButton);
 
     connect(ui->jackAutospawnCheckBox, &QCheckBox::stateChanged, this, &MainWindow::enableApplyButton);
+    connect(ui->jackConnectPortsCheckBox, &QCheckBox::stateChanged, this, &MainWindow::enableApplyButton);
     connect(ui->jackBufferSizeSlider, &QSlider::valueChanged, this, &MainWindow::updateJackBufferSizeEdit);
     connect(ui->jackBufferSizeLine, &QLineEdit::editingFinished, this, &MainWindow::updateJackBufferSizeSlider);
 
@@ -474,10 +475,9 @@ MainWindow::MainWindow(QWidget *parent) :
     for(int i = 0;backendList[i].backend_name[0];i++)
     {
         QList<QListWidgetItem*> items = ui->backendListWidget->findItems(
-            backendList[i].full_string, Qt::MatchFixedString
-        );
-        foreach(const QListWidgetItem *item, items)
-            ui->backendListWidget->setItemHidden(item, false);
+            backendList[i].full_string, Qt::MatchFixedString);
+        foreach(QListWidgetItem *item, items)
+            item->setHidden(false);
     }
 
     loadConfig(getDefaultConfigName());
@@ -918,6 +918,7 @@ void MainWindow::loadConfig(const QString &fname)
     ui->pulseAdjLatencyCheckBox->setCheckState(getCheckState(settings.value("pulse/adjust-latency")));
 
     ui->jackAutospawnCheckBox->setCheckState(getCheckState(settings.value("jack/spawn-server")));
+    ui->jackConnectPortsCheckBox->setCheckState(getCheckState(settings.value("jack/connect-ports")));
     ui->jackBufferSizeLine->setText(settings.value("jack/buffer-size", QString()).toString());
     updateJackBufferSizeSlider();
 
@@ -1127,6 +1128,7 @@ void MainWindow::saveConfig(const QString &fname) const
     settings.setValue("pulse/adjust-latency", getCheckValue(ui->pulseAdjLatencyCheckBox));
 
     settings.setValue("jack/spawn-server", getCheckValue(ui->jackAutospawnCheckBox));
+    settings.setValue("jack/connect-ports", getCheckValue(ui->jackConnectPortsCheckBox));
     settings.setValue("jack/buffer-size", ui->jackBufferSizeLine->text());
 
     settings.setValue("alsa/device", ui->alsaDefaultDeviceLine->text());
